@@ -9,7 +9,7 @@ frameworks into the ae namespace.
 The plan is to integrate the following GUI frameworks until the begin of 2021:
 
 * :mod:`Kivy <ae.kivy_app>` - see also :ref:`kivy lisz demo app <https://gitlab.com/ae-group/kivy_lisz>`
-* :mod:`Enaml <ae.enaml_app>`
+* :mod:`Enaml <ae.enaml_app>` - see also :ref:`enaml lisz demo app <https://gitlab.com/ae-group/enaml_lisz>`
 * :mod:`Beeware <ae.beeware_app>`
 * :mod:`pyglet <ae.pyglet_app>`
 * :mod:`pygobject <ae.pygobject_app>`
@@ -48,45 +48,44 @@ integrate new gui framework
 
 For to integrate a new Python GUI framework you have to declare a
 new class that inherits from :class:`MainAppBase` and implements at
-least the two abstract methods :meth:`~MainAppBase.app_init`
-and :meth:`~MainAppBase.run_app`.
+least the abstract method :meth:`~MainAppBase.init_app`.
 
 Most GUI frameworks are providing an application class that need to
 be initialized. For to keep a reference to the framework app class
 instance within your main app class you can use the
 :attr:`~MainAppBase.framework_app` attribute of :class:`MainAppBase`.
 
+If you want ot use the :attr:`~MainAppBase.root_win` and/or
+:attr:`~MainAppBase.root_layout` attributes you can initialize them
+also within :meth:`~MainAppBase.init_app`.
+
 .. hint::
-    The initialization of :attr:`~MainAppBase.framework_app` is optional.
-    Alternatively you could also add the framework application as mixin
+    The initialization of the attributes :attr:`~MainAppBase.framework_app`
+    :attr:`~MainAppBase.root_win` and :attr:`~MainAppBase.root_layout` is optional.
+    You could e.g. also add the framework application as mixin
     to the main app class.
 
-A typical implementation of a framework-specific main app class could
-look like::
+A typical implementation of a framework-specific main app class
+looks like::
 
     from new_gui_framework import NewFrameworkApp
 
     class NewFrameworkMainApp(MainAppBase):
-        def app_init(self):
+        def init_app(self):
             self.framework_app = NewFrameworkApp()
+            self.root_win = MainView()
+            self.root_layout = self.root_win.main_layout
 
-        def run_app(self):
-            self.framework_app.run()
-            # next statement is optional, for to pass the app exit code to the OS
-            sys.exit(self._exit_code)
+            return self.framework_app.start, self.framework_app.stop
 
-Both implementations of these abstract methods will be executed only once
-at app startup. Initialize within the first executed method
-:meth:`~MainAppBase.app_init` the GUI framework and
-prepare it for the app startup. The :meth:`~MainAppClass.run_app`
-method will be called from the main module of your app project for to
-start the framework app instance.
+:meth:`~MainAppBase.init_app` will be executed only once
+at app startup for to initialize the GUI framework and
+prepare it for the app startup.
 
-If you want ot use the :attr:`~MainAppBase.root_win` and/or
-:attr:`~MainAppBase.root_layout` attributes you can initialize them
-either within :meth:`~MainAppBase.app_init`. At latest they
-should be initialized just before or in the execution of
-:meth:`~MainAppClass.run_app`.
+For to finally startup the app the :meth:`~MainAppClass.run_app` method
+has to be called from the main module of your app project.
+:meth:`~MainAppBase.run_app` will then start the GUI event loop by
+calling the first method that got returned by :meth:`~MainAppBase.init_app`.
 
 
 key press event dispatching
@@ -255,7 +254,7 @@ of the main app instance).
 """
 from abc import ABC, abstractmethod
 from configparser import NoSectionError
-from typing import Any, Dict, Tuple, List, Optional
+from typing import Any, Callable, Dict, Tuple, List, Optional
 
 from ae.updater import check_all                    # type: ignore
 from ae.console import ConsoleApp                   # type: ignore
@@ -263,7 +262,7 @@ from ae.console import ConsoleApp                   # type: ignore
 from ae.files import FilesRegister, RegisteredFile  # type: ignore
 
 
-__version__ = '0.0.13'
+__version__ = '0.0.14'
 
 
 AppStateType = Dict[str, Any]                   #: app state config variable type
@@ -311,24 +310,28 @@ class MainAppBase(ConsoleApp, ABC):
 
         :param console_app_kwargs:  kwargs to be passed to the __init__ method of :class:`~ae.console_app.ConsoleApp`.
         """
-        self._exit_code = 0     # init by stop_app() and passed onto OS by run_app()
+        self._exit_code: int = 0            #: init by stop_app() and passed onto OS by run_app()
+        self._start_event_loop: Callable    #: callable to start event loop of GUI framework
+        self._stop_event_loop: Callable     #: callable to start event loop of GUI framework
 
         self.context_path = list()  # init for Literal type recognition - will be overwritten by setup_app_states()
         super().__init__(**console_app_kwargs)
         self.load_app_states()
         self.load_images()
         self.load_sounds()
-        self.app_init()
+
+        self._start_event_loop, self._stop_event_loop = self.init_app()
+        assert callable(self._start_event_loop) and callable(self._stop_event_loop), \
+            f"MainAppBase.__init__: start/stop event handler methods have to be callable"
 
     # abstract methods
 
     @abstractmethod
-    def app_init(self):
-        """ callback to framework api for to initialize an app instance. """
+    def init_app(self) -> Tuple[Callable, Callable]:
+        """ initialize framework app instance and root window/layout, return GUI event loop start/stop methods.
 
-    @abstractmethod
-    def run_app(self) -> str:
-        """ startup main and framework applications. """
+        :return:        tuple of two callable, the 1st for to start and the 2nd for to stop/exit the GUI event loop.
+        """
 
     # base implementation helper methods (can be overwritten by framework portion or by user main app)
 
@@ -353,6 +356,7 @@ class MainAppBase(ConsoleApp, ABC):
         """
         event_callback = getattr(self, method, None)
         if event_callback:
+            assert callable(event_callback), f"MainAppBase.call_event: {method!r} is not callable ({args}, {kwargs})"
             return event_callback(*args, **kwargs)
         return None
 
@@ -484,6 +488,18 @@ class MainAppBase(ConsoleApp, ABC):
 
         return app_state
 
+    def run_app(self):
+        """ startup main and framework applications. """
+        self.dpo(f"MainAppBase.run_app")
+
+        if not self._parsed_args:
+            self._parse_args()
+
+        try:
+            self._start_event_loop()
+        finally:
+            self.shutdown(self._exit_code or None)  # don't call sys.exit() for zero exit code
+
     def save_app_states(self) -> str:
         """ save app state in config file.
 
@@ -528,12 +544,14 @@ class MainAppBase(ConsoleApp, ABC):
 
         :param exit_code:   optional exit code.
         """
-        self.dpo(f"stop_app {exit_code}")
+        self.dpo(f"MainAppBase.stop_app {exit_code}")
         self._exit_code = exit_code
 
         if self.root_win:
             # neither self.framework_app.stop() nor self.framework_app._qapp.exit(exit_code) trigger window closed event
-            self.root_win.close()
+            self.root_win.close()   # close window to save app state data
+
+        self._stop_event_loop()     # will exit the self._start_event_loop() method called by self.run_app()
 
     def _update_observable_app_states(self, app_state: AppStateType):
         """ update all the observable app states.
